<?php

namespace Intergo\Providers\MessageManager;

use Intergo\Providers\MessageManager\Drivers\AlarisDriver;
use Intergo\Providers\MessageManager\Drivers\JasminDriver;
use Intergo\Providers\MessageManager\Drivers\RouteeDriver;
use Intergo\Providers\MessageManager\Drivers\RouteeOtpDriver;
use Intergo\Providers\MessageManager\Drivers\RouteeViberDriver;
use Intergo\Providers\MessageManager\Drivers\ItsmppDriver;
use Intergo\Providers\MessageManager\Drivers\NthViberDriver;
use Illuminate\Support\Manager;

use function GuzzleHttp\Promise\all;

class SMSManager extends Manager
{
    /**
     * The message to send
     *
     * @var Message
     */
    protected $message;

    /**
     * @param $provider
     * @param array $message
     * @return mixed
     */
    public function channel($provider, array $message)
    {
        $this->message = $message;
        if(is_string($provider))
        {
            $driver = $this->driver($provider);
        }
        elseif($provider->is_jasmin_provider)
        {
            $driver = $this->driver('jasmin')->config($provider);
        }
        else
        {
            $driver = $this->driver($provider->slug);
        }
        return $driver;
    }

    /**
     * Get the default driver name.
     *
     * @return string
     */
    public function getDefaultDriver()
    {
        return 'routee';
    }

    public function createRouteeDriver(): RouteeDriver
    {
        return new RouteeDriver();
    }

    public function createRouteeOtpDriver(): RouteeOtpDriver
    {
        return new RouteeOtpDriver();
    }

    public function createJasminDriver(): JasminDriver
    {
        return new JasminDriver();
    }

    public function createViberDriver() //RouteeViberDriver|NthViberDriver #This is only availablein php v8
    {
        if ($this->message['viber_provider'] == 'nth') {
           return new NthViberDriver();
        }
        return new RouteeViberDriver();
    }

    /**
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPP Mock Driver to the correct driver and separate the queue
     * of mock from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppMockDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPP OTP Driver to the correct driver and separate the queue
     * of OTP from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppOtpDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPPD Driver to the correct driver and separate the queue
     * of D from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppdDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPPS Driver to the correct driver and separate the queue
     * of S from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppsDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPPW Driver to the correct driver and separate the queue
     * of W from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppwDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPPDL Driver to the correct driver and separate the queue
     * of DL from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmppdlDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPPP Driver to the correct driver and separate the queue
     * of P from the actual itsmpp provider
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return ItsmppDriver
     */
    public function createItsmpppDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPP HQ Driver to the correct driver and separate the queue
     * of HQ from the actual itsmpp provider
     *
     * @return ItsmppDriver
     */
    public function createItsmppHqDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the ITSMPP OTP Global Driver to the correct driver and separate the queue
     * of OTP GLOBAL from the actual itsmpp provider
     *
     * @return ItsmppDriver
     */
    public function createItsmppOtpGlobalDriver(): ItsmppDriver
    {
        return new ItsmppDriver();
    }

    /**
     * This is to point the Alaris OTP Global Driver to the correct driver and separate the queue
     * of OTP from the actual Alaris provider
     *
     * @return ItsmppDriver
     */
    public function createAlarisOtpDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris Mock Driver to the correct driver and separate the queue
     * of mock from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlarisMockDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris D Driver to the correct driver and separate the queue
     * of d from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlarisdDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris DL Driver to the correct driver and separate the queue
     * of dl from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlarisdlDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris S Driver to the correct driver and separate the queue
     * of s from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlarissDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris W Driver to the correct driver and separate the queue
     * of w from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlariswDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }

    /**
     * This is to point the Alaris G Driver to the correct driver and separate the queue
     * of G from the actual Alaris provider
     *
     * @return AlarisDriver
     */
    public function createAlarisgDriver(): AlarisDriver
    {
        return new AlarisDriver();
    }
}
